//*********************************************************
// www.Ultimategameprogramming.com
// Per-Vertex Lighting shader demo.
// Created by the Programming Ace.
//
// Input Variables used:
//    modelViewProj  - Holds the model view projection matrix.
//    modelView      - Holds model view matrix.
//    lightColor     - Holds light color.
//    lightPos       - Holds light position.
//    ambientMat     - Ambient material.
//    diffuseMat     - Diffuse material.
//    ambientCol     - Ambient color.
//    decal          - Holds the texture.
//
//*********************************************************


// Shader variables.  The first is binded to the WorldViewProjection register.
float4x4 modelViewProj : WorldViewProjection;
float4x4 modelView : WorldView;
float4 lightColor;
float4 lightPos;
float4 ambientMat;
float4 ambientCol;
float4 diffuseMat;

texture decal;

sampler Sampler = sampler_state
{
    Texture   = (decal);
};


// Vertex input.
struct App_Input
{
   float3 vertexPos  : POSITION;
   float3 normal     : NORMAL;
   float2 texture0   : TEXCOORD0;
};

struct Sphere_Input
{
   float3 vertexPos  : POSITION;
};

// Vertex output.
struct Vs_Output
{
   float4 vertexPos  : POSITION;
   float2 texture0   : TEXCOORD0;
   float4 color      : COLOR0;
};

struct Sphere_Output
{
   float4 vertexPos  : POSITION;
   float4 color      : COLOR0;
};

// Pixel output.
struct Ps_Output
{
	float4 color      : COLOR;
};


// The vertex shader...
Vs_Output PerVertexLightVS(App_Input IN)
{
   Vs_Output vs_out;

   float4 pos = mul(modelView, float4(IN.vertexPos, 1));

   vs_out.vertexPos = mul(modelViewProj, float4(IN.vertexPos, 1));
   vs_out.texture0 = IN.texture0;
   
   // Calculate the ambient color.
   float3 ambient = ambientMat * ambientCol;

   // Calculate the diffuse color.
   float3 lightVec = normalize(lightPos - pos);
   float dp = max(dot(IN.normal, lightVec), 0);
   float3 diffuse = diffuseMat * lightColor * dp;

   // Calculate the final color as ambient + diffuse.
   vs_out.color.xyz = ambient + diffuse;
   vs_out.color.w = 1;
   
   return vs_out;
}


// The pixel shader...
Ps_Output PerVertexLightPS(Vs_Output IN)
{
   Ps_Output ps_out;

   ps_out.color = tex2D(Sampler, IN.texture0) * IN.color;
   return ps_out;
}


Sphere_Output LightEffectVS(Sphere_Input IN)
{
   Sphere_Output vs_out;

   vs_out.vertexPos = mul(modelViewProj, float4(IN.vertexPos, 1));
   vs_out.color = lightColor;
   
   return vs_out;
}

Ps_Output LightEffectPS(Sphere_Output IN)
{
   Ps_Output ps_out;

   ps_out.color = IN.color;
   return ps_out;
}


technique PerVertexLighting
{
   pass Pass0
   {
      Sampler[0] = (Sampler);

      // Here we will compile and set the vertex and pixel shader.
      VertexShader = compile vs_2_0 PerVertexLightVS();
      PixelShader  = compile ps_2_0 PerVertexLightPS();
   }
}

technique SimpleLight
{
   pass Pass0
   {
      // Here we will compile and set the vertex and pixel shader.
      VertexShader = compile vs_2_0 LightEffectVS();
      PixelShader  = compile ps_2_0 LightEffectPS();
   }
}